#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include "math.h"

/*
	General purpose fast fourier transform (FFT) package.

	Copyright (c) 1999, Rich Drewes

	Code is possibly inspired by Derenzo's book "Interfacing".
*/


/*
  meaning of output array is a function of meaning of input array
  (clearly!).  if nu is 10, 2^10=FFTPOINTS, only first FFTPOINTS entries are
  significant.  output is reflected about halfway pt. 

  if input is 8000 hz., when we do analysis we can recognize freqs up
  to 4000 hz.

  512 is 4000 hz.

  performance:
floating point
	with precomputing sines and cosines array, got 100 ffts in 7 secs
	.07 secs per 70 ms per, at 8Ks/sec, that is 560 samples in the
	time to compute one fft.  not too hot.
	(got 100 ffts in 11 secs without precomputing)

fixed point
	surprisingly, not much faster!

*/

#define NOISESKIP	4	/* don't compute extremely low or high freq
					components in energy (noisy) */


float fftprintenergy(float xr[], float xi[], int fftpoints)
{
    int i;
    float	max=0.0;
    float v, sum, avg;
    int	maxindex=0;

    for(i=0; i<(fftpoints/2); i++) {
		v=xr[i] * xr[i] + xi[i] * xi[i];
		printf("%d\n", (int) v);
    }
}

/*
  nu is number of bits?  2^FFTBITS= FFTPOINTS
*/
#define TWOPI 6.283185307
#define PITWO 1.570796327

//float stab[FFTPOINTS+1], ctab[FFTPOINTS+1];
float *stab=NULL, *ctab=NULL;
int n, nu1, n2, n1;

fftinit(nu)
int nu;
{
    int i;
    float arg;

    n=1<<nu;

	if(!stab) {
		free(stab);
		stab=(float *) malloc(sizeof(float) * n);
	}

	if(!ctab) {
		free(ctab);
		ctab=(float *) malloc(sizeof(float) * n);
	}

    for(i=0; i<n; ++i) {
		arg=TWOPI*i/n;
		stab[i]=sin(arg);
		ctab[i]=sin(arg+PITWO);
    }
    return(1);
}

// computation is done in place
fft(xr, xi, nu, ie)
float xr[], xi[];
int nu;
int ie;
{
    int p, k1n2, i, k, l;
    float c, s, tr, ti;


    n=1<<nu;
    n2=n/2;
    nu1=nu-1;
    k=0;
    for(l=0; l<nu; ++l) {
        while(k<n) {
	    for(i=0; i<n2; ++i) {
		n1=1<<nu1;
		p=ibitr(k/n1, nu);
		s=stab[p];
		c=ctab[p];
		if(ie>0) s=-s;
		k1n2=k+n2;
		tr=xr[k1n2]*c+xi[k1n2]*s;
		ti=xi[k1n2]*c-xr[k1n2]*s;
		xr[k1n2]=xr[k]-tr;
		xi[k1n2]=xi[k]-ti;
		xr[k]=xr[k]+tr;
		xi[k]=xi[k]+ti;
		k=k+1;
	    }
	    k=k+n2;
	}
	k=0;
	nu1=nu1-1;
	n2=n2/2;
    }
    for(k=0; k<n; ++k) {
	i=ibitr(k, nu);
	if(i>k) {
	    tr=xr[k];
	    ti=xi[k];
	    xr[k]=xr[i];
	    xi[k]=xi[i];
	    xr[i]=tr;
	    xi[i]=ti;
	}
    }
    if(ie>0) {
	for(i=0; i<n; ++i) {
	    xr[i]=xr[i]/n;
	    xi[i]=xi[i]/n;
	}
    }
    return(1);
}

int ibitr(j, nu)
int j, nu;
{
    int rm, lm, i, bitr;

    rm=1;
    lm=1<<(nu-1);
    bitr=0;
    for(i=0; i<nu; ++i) {
	if((j&rm)!=0) bitr=bitr|lm;
	rm=rm<<1;
	lm=lm>>1;
    }
    return(bitr);
}
